const fs = require('fs');
const readline = require('readline');

const faq = {
  groups: [
    {
      name: 'FAQs',
      anchor: "general",
      subgroups: []
    }
  ]
};

async function parseFaq() {
  const fileStream = fs.createReadStream('scripts/faq.md');
  const rl = readline.createInterface({
    input: fileStream,
    crlfDelay: Infinity
  });

  let lastGroup = 0;
  let lastSubgroup = 0;
  let lastItem = 0;
  for await (const line of rl) {
    // console.log(`Line from file: ${line}`);

    if (line.startsWith('# ')) {
      const name = line.substring(2);

      // new subgroup (should be group, but as of 22-03-22 there is only 1 group in most of md doc)
      lastItem = 0;
      faq
        .groups[lastGroup]
        .subgroups
        .push({ name, anchor: slugify(name), items: [] });

      lastSubgroup = faq.groups[lastGroup].subgroups.length - 1;
      // console.log('subgroup, item: ', lastSubgroup, lastItem);
    } else if (line.startsWith('## ')) {
      // should be subgroup, but as of 22-03-22 there is only 1 group in most of md doc)
    } else if (line.startsWith('### ')) {
      const title = line.substring(4);

      // new item
      faq
        .groups[lastGroup]
        .subgroups[lastSubgroup]
        .items
        .push({ title, content: '' });

      lastItem = faq.groups[lastGroup].subgroups[lastSubgroup].items.length - 1;
      // console.log('subgroup, item: ', lastSubgroup, lastItem);
    } else if (!line.startsWith('[')) {
      let newLine = line;

      // look for markdown URLs
      const mdRegex = /\[([^\[]+)\]\((.*?)\)/;
      const mdUrl = line.match(mdRegex);
      if (mdUrl) {
        console.log('mdUrl: ', mdUrl);
        newLine = newLine.replace(mdUrl[0],
          ` <Link to={'${mdUrl[2]}'}>${mdUrl[1]}</Link> `);
      } else {
        // look for raw URLs
        const urlRegex = /(https?:\/\/[^ ]*)/;
        const url = newLine.match(urlRegex);
        if (url) {
          newLine = newLine.replace(url[1],
            ` <Link to={'${url[1]}'}>${url[1]}</Link> `);
        }
      }

      // everything else is text of the last item
      let item = faq.groups[lastGroup].subgroups[lastSubgroup].items[lastItem];
      // console.log('item: ', item);
      faq
        .groups[lastGroup]
        .subgroups[lastSubgroup]
        .items[lastItem]
        .content = item.content + newLine;
    }
  }

  // console.log(faq.groups[0]);
  let data = JSON.stringify(faq, null, 2);
  fs.writeFileSync('scripts/faq.json', data);
}

function slugify(value) {
  return value
    .normalize('NFD') // split an accented letter in the base letter and the acent
    .replace(/[\u0300-\u036f]/g, '') // remove all previously split accents
    .toLowerCase()
    .trim()
    .replace(/[^a-z0-9 ]/g, '') // remove all chars not letters, numbers and spaces (to be replaced)
    .replace(/\s+/g, '-') // separator
}

parseFaq();
